class PasswordsHandbookAdminSettingsService {

    /**
     *
     * @param setting
     * @param value
     * @returns {Promise}
     */
    set(setting, value) {
        return this._createRequest(`settings/${setting}`, {value}, 'PUT');
    }

    // noinspection JSMethodCanBeStatic
    /**
     * Creates an api request
     *
     * @param path
     * @param data
     * @param method
     * @returns {Promise}
     * @private
     */
    async _createRequest(path, data = null, method = 'GET') {
        let options = {method, credentials: 'include'};

        options.headers = new Headers();
        options.headers.append('Accept', `application/json, text/plain, */*`);
        options.headers.append('requesttoken', OC.requestToken);

        if(data) {
            if(method === 'GET') options.method = 'POST';
            options.headers.append('Content-Type', 'application/json');
            options.body = JSON.stringify(data);
        }

        try {
            let url      = OC.generateUrl(`/apps/passwords_handbook/admin/${path}`),
                response = await fetch(new Request(url, options), options);
            return await PasswordsHandbookAdminSettingsService._processResponse(response);
        } catch(e) {
            console.error('Request failed', e);
            throw e;
        }
    }

    /**
     *
     * @param response
     * @returns {Promise<*>}
     * @private
     */
    static async _processResponse(response) {
        if(!response.ok) {
            console.error(response);
            throw new Error('Api request failed');
        }

        let contentType = response.headers.get('content-type');
        if(contentType && contentType.indexOf('application/json') !== -1) {
            return await response.json();
        } else {
            console.error('Invalid response format', response);
            throw new Error('Invalid response format');
        }
    }
}

class PasswordsHandbookAdminSettings {
    constructor() {
        this._timer = {success: null, error: null};
        this.api = new PasswordsHandbookAdminSettingsService();
    }

    initialize() {
        document
            .querySelectorAll('[data-handbook-setting]')
            .forEach(
                (el) => {
                    el.addEventListener(
                        'change',
                        (e) => {
                            let target = e.target,
                                key    = target.dataset.handbookSetting,
                                value  = target.value;

                            if(target.getAttribute('type') === 'checkbox') {
                                value = target.checked;
                            }

                            this.api.set(key, value)
                                .then((d) => {
                                    this._showMessage('saved', `[data-handbook-setting="${key}"]`);
                                    if(d.hasOwnProperty('url')) {
                                        document.getElementById('passwords-handbook-url').value = d.url;
                                    }
                                    if(d.hasOwnProperty('error')) {
                                        let el = document.getElementById('passwords-handbook-error');
                                        if(d.error === null) {
                                            el.style.display = 'none';
                                        } else {
                                            el.innerText = d.error;
                                            el.style.display = 'block';
                                        }
                                    }
                                })
                                .catch((d) => {
                                    this._showMessage('error', `[data-handbook-setting="${key}"]`);
                                    if(d.message) OC.Notification.show(PasswordsHandbookAdminSettings._translate(d.message));
                                });
                        }
                    );
                }
            );
    }

    /**
     * Show save success/fail message
     *
     * @param type
     * @param target
     * @private
     */
    _showMessage(type, target) {
        let el = document.querySelector(`section.passwords ${target}`).closest('form').querySelector(`h3 .response.${type}`);
        el.classList.remove('active');
        el.classList.add('active');

        clearTimeout(this._timer[type]);
        this._timer[type] = setTimeout(
            () => {
                el.classList.remove('active');
                if(type === 'error') location.reload();
            },
            1000);
    }

    static _translate(text) {
        return OC.L10N.translate('passwords', text);
    }
}

window.addEventListener('DOMContentLoaded', () => {
    let PwHandbookSettings = new PasswordsHandbookAdminSettings();
    PwHandbookSettings.initialize();
});