<?php
/*
 * @copyright 2023 Passwords Handbook App
 *
 * @author Marius David Wieschollek
 * @license AGPL-3.0
 *
 * This file is part of the Passwords App
 * created by Marius David Wieschollek.
 */

namespace OCA\PasswordsHandbook\Controller\Admin;

use JetBrains\PhpStorm\ArrayShape;
use OCA\Passwords\Exception\ApiException;
use OCA\PasswordsHandbook\Service\FeaturesService;
use OCA\PasswordsHandbook\Service\HandbookUrlService;
use OCA\PasswordsHandbook\Service\SetupCheckService;
use OCP\AppFramework\ApiController;
use OCP\AppFramework\Http\JSONResponse;
use OCP\IRequest;

class SettingsController extends ApiController {

    /**
     * @param string             $appName
     * @param IRequest           $request
     * @param HandbookUrlService $handbookUrlService
     * @param SetupCheckService  $setupCheckService
     * @param FeaturesService    $featuresService
     */
    public function __construct(
        string                       $appName,
        IRequest                     $request,
        protected HandbookUrlService $handbookUrlService,
        protected SetupCheckService  $setupCheckService,
        protected FeaturesService    $featuresService
    ) {
        parent::__construct(
            $appName,
            $request,
            'PUT, GET, DELETE'
        );
    }

    /**
     * @return JSONResponse
     */
    public function index(): JSONResponse {
        $settings = [
            $this->makeSetting('enabled', $this->handbookUrlService->isUrlSet())
        ];

        return new JSONResponse($settings);
    }

    /**
     * @param $id
     *
     * @return JSONResponse
     */
    public function show($id): JSONResponse {
        $data = [];
        if($id === 'enabled') {
            $data = $this->makeSetting('enabled', $this->handbookUrlService->isUrlSet());
        }

        return new JSONResponse($data);
    }

    /**
     * @param string $id
     * @param        $value
     *
     * @return JSONResponse
     */
    public function update(string $id, $value): JSONResponse {
        if($id === 'enabled') {
            if($value) {
                $this->handbookUrlService->setUrl();
            } else {
                $this->handbookUrlService->resetUrl();
            }
            $data = $this->makeSetting($id, $this->handbookUrlService->isUrlSet());
        } else if(str_starts_with($id, 'feature.')) {
            [, $client, $name] = explode('.', $id, 3);
            $this->featuresService->setFeature($client, $name, !empty($value));
            $isEnabled = $this->featuresService->getFeature($client, $name);

            $data = $this->makeSetting($id, $isEnabled);
        }

        $data['url']   = $this->handbookUrlService->getCurrentUrl();
        $data['error'] = $this->setupCheckService->check();

        return new JSONResponse($data);
    }

    /**
     * @param string $id
     *
     * @return JSONResponse
     */
    public function destroy(string $id): JSONResponse {
        if($id === 'enabled') {
            $this->handbookUrlService->resetUrl();
        }

        return $this->show($id);
    }

    /**
     * @param string $id
     * @param        $value
     *
     * @return array
     */
    #[ArrayShape(['name' => "", 'value' => ""])]
    protected function makeSetting(string $id, mixed $value): array {
        return [
            'name'  => $id,
            'value' => $value
        ];
    }
}