<?php
/*
 * @copyright 2023 Passwords Handbook App
 *
 * @author Marius David Wieschollek
 * @license AGPL-3.0
 *
 * This file is part of the Passwords App
 * created by Marius David Wieschollek.
 */

namespace OCA\PasswordsHandbook\Service;

use Exception;
use OCP\Http\Client\IClientService;
use OCP\IL10N;

class SetupCheckService {

    public function __construct(
        protected HandbookUrlService $handbookUrlService,
        protected IClientService $clientService,
        protected IL10N $language) {
    }

    /**
     * @return string|null
     */
    public function check(): ?string {
        $url = $this->handbookUrlService->getCurrentUrl();
        if(empty($url)) return $this->language->t('The handbook URL is empty.');
        if(!filter_var($url, FILTER_VALIDATE_URL) || !str_starts_with($url, 'https://')) {
            return $this->language->t('The handbook URL is not a valid URL.');
        }

        $client = $this->clientService->newClient();
        try {
            $response = $client->get($url.'Index.md', ['allow_redirects' => false]);
        } catch(Exception $e) {
            return $this->language->t('Unable to load Index.md: %s', $e->getMessage());
        }

        $status = $response->getStatusCode();
        if($status !== 200) {
            return $this->language->t('Invalid status for Index.md: HTTP %s', $status);
        }

        $contentType = strtolower($response->getHeader('Content-Type'));
        if(!str_starts_with($contentType, 'text/plain') && !str_starts_with($contentType, 'text/markdown')) {
            return $this->language->t('Index.md content type is not "text/plain" or "text/markdown": %s', $response->getHeader('Content-Type'));
        }

        if(empty(trim($response->getBody()))) {
            return $this->language->t('Index.md is empty');
        }

        return null;
    }
}